import { Reflector } from '@nestjs/core'
import {
  ExecutionContext,
  Injectable,
  Logger,
  UnauthorizedException,
} from '@nestjs/common'
import { AuthGuard } from '@nestjs/passport'
import 'rxjs'
import { NEED_AUTH_KEY } from '../decorators/need-auth.decorator'
import { JsonWebTokenError } from 'jsonwebtoken'

@Injectable()
export class JwtAuthGuard extends AuthGuard('jwt') {
  private readonly logger = new Logger(JsonWebTokenError.name)

  constructor(private reflector: Reflector) {
    super()
  }

  canActivate(context: ExecutionContext) {
    const needAuth = this.reflector.getAllAndOverride<boolean>(NEED_AUTH_KEY, [
      context.getHandler(),
      context.getClass(),
    ])
    if (needAuth) {
      return super.canActivate(context)
    }
    return true
  }

  handleRequest(err: Error, user, info: Error) {
    if (err || info || !user) {
      this.logger.error(err?.message || info?.message)
      throw new UnauthorizedException(err?.message || info?.message)
    }
    return user
  }
}
